#include <string.h>

#include <glib-object.h>

#include "gegl/gegl-types.h"
#include "gegl/gegl-pad.h"

#include "op_source.h"
#include "canvas/types.h"
#include "canvas/canvas_rect.h"
#include "canvas/canvas.h"

enum
{
  PROP_0,
  PROP_OUTPUT,
  PROP_LAST
};


static void  op_source_class_init (OpSourceClass *klass);
static void  op_source_init       (OpSource      *self);

static void     finalize     (GObject      *gobject);
static void     get_property (GObject      *gobject,
                              guint         prop_id,
                              GValue       *value,
                              GParamSpec   *pspec);
static void     set_property (GObject      *gobject,
                              guint         prop_id,
                              const GValue *value,
                              GParamSpec   *pspec);
static gboolean evaluate     (GeglOperation   *operation,
                              const gchar  *output_prop);
static void     associate    (GeglOperation *operation);



G_DEFINE_TYPE (OpSource, op_source, GEGL_TYPE_OPERATION)


static void
op_source_class_init (OpSourceClass * klass)
{
  GObjectClass       *gobject_class   = G_OBJECT_CLASS (klass);
  GeglOperationClass *operation_class = GEGL_OPERATION_CLASS (klass);

  gobject_class->set_property = set_property;
  gobject_class->get_property = get_property;
  gobject_class->finalize     = finalize;

  operation_class->evaluate = evaluate;
  operation_class->associate = associate;

  g_object_class_install_property (gobject_class, PROP_OUTPUT,
                                   g_param_spec_object ("output",
                                                        "Output",
                                                        "An output property",
                                                        CANVAS_TYPE_RECT,
                                                        G_PARAM_READABLE |
                                                        GEGL_PAD_OUTPUT));

}

static void
op_source_init (OpSource *self)
{
  self->output = NULL;
}

static void
associate (GeglOperation *self)
{
  GeglOperation   *operation = GEGL_OPERATION (self);
  GObjectClass *object_class = G_OBJECT_GET_CLASS (self);

  gegl_operation_create_pad (operation,
                               g_object_class_find_property (object_class,
                                                             "output"));
}

static void
finalize (GObject *object)
{
  OpSource *self = OP_SOURCE (object);

  if (self->output)
    g_object_unref (self->output);

  G_OBJECT_CLASS (op_source_parent_class)->finalize (object);
}

static void
get_property (GObject      *object,
              guint         prop_id,
              GValue       *value,
              GParamSpec   *pspec)
{
  OpSource *self = OP_SOURCE (object);

  switch (prop_id)
  {
    case PROP_OUTPUT:
      g_value_set_object(value, self->output);
      break;
    default:
      break;
  }
}

static void
set_property (GObject      *object,
              guint         prop_id,
              const GValue *value,
              GParamSpec   *pspec)
{
  OpSource *self = OP_SOURCE (object);
  self = NULL;

  switch (prop_id)
  {
    default:
      break;
  }
}

#include <stdio.h>

static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpSourceClass *klass     = OP_SOURCE_GET_CLASS (operation);
  gboolean success;

  success = klass->evaluate (operation, output_prop);

  return success;
}
