#ifdef MAGIC_SELF
 
magic_int    (x,   0,   G_MAXINT,  0)
magic_int    (y,   0,   G_MAXINT,  0)

#else

#define MAGIC_SELF                 "op_over.c"
#define MAGIC_SUPER_CLASS_COMPOSER
#define MAGIC_TypeName             OpOver
#define MAGIC_TypeNameClass        OpOverClass
#define MAGIC_type_name            op_over
#define MAGIC_type_name_init       op_over_init
#define MAGIC_type_name_class_init op_over_class_init
#include "magic.h"

#include <stdio.h>

static void
copy_rect (CanvasRect *src,
           CanvasRect *dst)
{
    gint tile_x, tile_y;
    for (tile_y = 0; tile_y < tile_tile (dst->height) + 1; tile_y++)
      {
        for (tile_x = 0; tile_x < tile_tile (dst->width) + 1; tile_x++)
          {
            Tile *tile_src = canvas_get_tile (src->canvas,
                  tile_tile (src->x) + tile_x,
                  tile_tile (src->y) + tile_y);
            Tile *tile = canvas_get_tile (dst->canvas, 
                  tile_tile (dst->x) + tile_x,
                  tile_tile (dst->y) + tile_y);
            tile_cpy (tile_src, tile);
          }
      }
}

static void
over_rect (CanvasRect *src,
           CanvasRect *aux,
           gint        x,
           gint        y);

/* Actual image processing code
 ************************************************************************/
static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpComposer *composer = OP_COMPOSER(operation);
  OpOver     *self     = (OpOver*)(composer);

  CanvasRect *input  = composer->input;
  CanvasRect *aux    = composer->aux;
  CanvasRect *output;

  if(strcmp("output", output_prop))
    return FALSE;

  if (composer->in_place)
    {
       g_warning ("not able to do inplace  ..");
       return FALSE;
    }
  else
    {
      output = CANVAS_RECT (canvas_alloc (input->canvas, input->width, input->height));
      copy_rect (input, output);
      over_rect (output, aux, self->x, self->y);

      self = NULL;
      aux = NULL;

      if (composer->output)
        g_object_unref (composer->output);
      composer->output = output;
    }
  return  TRUE;
}

static void
over_rect (CanvasRect *src,
           CanvasRect *aux,
           gint        x,
           gint        y)
{
  gint pix_x = x;
  gint pix_y = y;

  guchar *src_buf;
  guchar *aux_buf;

  src_buf = g_malloc0 (src->width * src->height * 4);
  if (!src_buf)
    {
      return;
    }
  aux_buf = g_malloc0 (aux->width * aux->height * 4);
  if (!aux_buf)
    {
      g_free (src_buf);
      return;
    }

  canvas_rect_export_buf (src, src_buf);
  canvas_rect_export_buf (aux, aux_buf);

  {
    gint y,x;
    gint src_rowstride = src->width * 4;
    gint aux_rowstride = aux->width * 4;

    for (y=0;y<src->height;y++)
      for (x=0;x<src->width;x++)
        {
          gint a_y = y-pix_y;
          gint a_x = x-pix_x;

          if (a_y >= 0 &&
              a_x >= 0 &&
              a_y < aux->height &&
              a_x < aux->width)
            {
              gint r,g,b,a;

              r = aux_buf[ a_y*aux_rowstride+a_x * 4 + 0];
              g = aux_buf[ a_y*aux_rowstride+a_x * 4 + 1];
              b = aux_buf[ a_y*aux_rowstride+a_x * 4 + 2];
              a = aux_buf[ a_y*aux_rowstride+a_x * 4 + 3];

              src_buf[ y*src_rowstride+x * 4 + 0] = r;
              src_buf[ y*src_rowstride+x * 4 + 1] = g;
              src_buf[ y*src_rowstride+x * 4 + 2] = b;
              src_buf[ y*src_rowstride+x * 4 + 3] = a;
            }
        }
  }
  
  canvas_rect_import_buf (src, src_buf);

  g_free (src_buf);
  g_free (aux_buf);
}

#endif
