#include <string.h>

#include <glib-object.h>

#include "gegl/gegl-types.h"
#include "gegl/gegl-pad.h"

#include "op_filter.h"
#include "op_point_filter.h"
#include "canvas/types.h"
#include "canvas/canvas_rect.h"
#include "canvas/canvas.h"

enum
{
  PROP_0,
  PROP_OUTPUT,
  PROP_INPUT,
  PROP_IN_PLACE,
  PROP_LAST
};

static void     op_point_filter_class_init (OpPointFilterClass *klass);

static void     op_point_filter_init       (OpPointFilter      *self);

static gboolean evaluate                   (GeglOperation *operation,
                                            const gchar   *output_prop);

G_DEFINE_TYPE (OpPointFilter, op_point_filter, TYPE_OP_FILTER)


static void
op_point_filter_class_init (OpPointFilterClass * klass)
{
  OpFilterClass *filter_class = OP_FILTER_CLASS (klass);
  filter_class->evaluate = evaluate;
}

static void
op_point_filter_init (OpPointFilter *self)
{
}

static void
duplicate (CanvasRect *src,
           CanvasRect *dst)
{
    gint tile_x, tile_y;
    for (tile_y = 0; tile_y < tile_tile (dst->height) + 1; tile_y++)
      {
        for (tile_x = 0; tile_x < tile_tile (dst->width) + 1; tile_x++)
          {
            Tile *tile_src = canvas_get_tile (src->canvas,
                  tile_tile (src->x) + tile_x,
                  tile_tile (src->y) + tile_y);
            Tile *tile = canvas_get_tile (dst->canvas, 
                  tile_tile (dst->x) + tile_x,
                  tile_tile (dst->y) + tile_y);
            tile_cpy (tile_src, tile);
          }
      }
}

static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpFilter           *filter = OP_FILTER(operation);
  OpPointFilterClass *klass  = OP_POINT_FILTER_GET_CLASS (operation);

  CanvasRect *input  = filter->input;
  CanvasRect *output;

  if(strcmp("output", output_prop))
    return FALSE;

  g_assert (klass->evaluate);

  if (filter->in_place)
    {
      output = input;
      g_object_ref (output);
      {
        guchar *buf;
        buf = g_malloc0 (output->width * output->height * 4);
        if (!buf)
          {
            filter->output = NULL;
            return FALSE;
          }
        canvas_rect_export_buf (input, buf);
        klass->evaluate (operation, buf, output->width, output->height, output->width * 4);
        canvas_rect_import_buf (output, buf);
        g_free (buf);
      }
      return TRUE;
    }
  else
    { /* FIXME: handle in place using tile iterating as well,.
                handle edge cases better
       */
      output = CANVAS_RECT (canvas_alloc (input->canvas, input->width, input->height));
      {
        duplicate (input, output);
        {
          gint tile_x, tile_y;
          for (tile_y = 0; tile_y < tile_tile (output->height) + 1; tile_y++)
            {
              for (tile_x = 0; tile_x < tile_tile (output->width) + 1; tile_x++)
                {
                  Tile *tile = canvas_get_tile (output->canvas, 
                        tile_tile (output->x) + tile_x,
                        tile_tile (output->y) + tile_y);

                  tile_prepare_write (tile);
                  klass->evaluate (operation, tile_get_data (tile),
                     tile_get_width (tile), tile_get_height (tile), tile_get_width (tile) * 4);
                }
            }
        }
      }
      if (filter->output)
        g_object_unref (filter->output);
      filter->output = output;
    }
  return  TRUE;
}
