/*  horizon, a sketch, note and scribbling application.
    Copyright (C) 2006 Eero Tanskanen <yendor@nic.fi>
                       Øyvind Kolås <pippin@gimp.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
*/
#include <glib.h>
#include <math.h>
#include "view.h"
#include "widget.h"
#include "slider.h"

#include "config.h"

/* Changing this value doesn't work at this point. */
#define SLIDER_MAX  65536

struct _Slider
{
  Widget   widget;
  
  /* Always return value between min 0 and SLIDER_MAX */
  
  guint    value;
  guint    old_value;
  gpointer data;
  
  Allocation *allocation;
  
  void (*value_changed_function) (Slider   *slider,
		                  gpointer  data);
};

#include "gegl/gegl.h"

static void
render (View     *view,
        gpointer  data)
{
  Box    *box = (Box*) view;
  Slider *slider = (Slider*) view;
  Widget *widget = (Widget*) view;

  GeglGraph *gegl;
  GeglNode  *load;
  GeglNode  *rect;
  GeglNode  *prev;

  gint x = CANVAS_RECT (slider->allocation)->x;
  gint y = CANVAS_RECT (slider->allocation)->y;

  gint width = box_get_width (box);
  gint height = box_get_height (box);
  gint mini_box_size = (height / 4) | 1;
  gint bar_height = (height - (mini_box_size * 2)) * 6 / 10;
  gint value = slider->value;
   
  if (value == slider->old_value)
    {
      return;
    }
  slider->old_value = value;

  x = value * width / SLIDER_MAX;
  y = height / 2;

  gegl = g_object_new (GEGL_TYPE_GRAPH, NULL);
  load = gegl_graph_create_node (gegl,
    "operation", "OpFetch",
    "canvas",    view->canvas,
    "x",         CANVAS_RECT (slider->allocation)->x,
    "y",         CANVAS_RECT (slider->allocation)->y,
    "width",     width,
    "height",    height,
    NULL);

  rect = gegl_graph_create_node (gegl,
    "operation", "OpFillRect",
    "in-place",  TRUE,
    "x",         0,
    "y",         0,
    "width",     width,
    "height",    height,
    "red",       widget->background_color[0],
    "green",     widget->background_color[1],
    "blue",      widget->background_color[2],
    "alpha",     65536,
    NULL);
  gegl_node_connect (rect, "input", load, "output");
  prev = rect;

  rect = gegl_graph_create_node (gegl,
    "operation", "OpFillRect",
    "in-place",  TRUE,
    "x",         (x),
    "y",         (y-bar_height/2),
    "width",     1,
    "height",    bar_height,
    "red",       widget->primary_color[0],
    "green",     widget->primary_color[1],
    "blue",      widget->primary_color[2],
    "alpha",     65536,
    NULL);
  gegl_node_connect (rect, "input", prev, "output");
  prev = rect;
  
  rect = gegl_graph_create_node (gegl,
    "operation", "OpFillRect",
    "in-place",  TRUE,
    "x",         (x-mini_box_size/2),
    "y",         (y-(bar_height/2+(mini_box_size))),
    "width",     mini_box_size,
    "height",    mini_box_size,
    "red",       widget->primary_color[0],
    "green",     widget->primary_color[1],
    "blue",      widget->primary_color[2],
    "alpha",     65536,
    NULL);
  gegl_node_connect (rect, "input", prev, "output");
  prev = rect;
  
  rect = gegl_graph_create_node (gegl,
    "operation", "OpFillRect",
    "in-place",  TRUE,
    "x",         (x-(mini_box_size-2)/2),
    "y",         (y-(bar_height/2+(mini_box_size-1))),
    "width",     (mini_box_size-2),
    "height",    (mini_box_size-2),
    "red",       widget->secondary_color[0],
    "green",     widget->secondary_color[1],
    "blue",      widget->secondary_color[2],
    "alpha",     65536,
    NULL);
  gegl_node_connect (rect, "input", prev, "output");
  prev = rect;

  rect = gegl_graph_create_node (gegl,
    "operation", "OpFillRect",
    "in-place",  TRUE,
    "x",         (x-(mini_box_size)/2),
    "y",         (y+(bar_height/2)),
    "width",     (mini_box_size),
    "height",    (mini_box_size),
    "red",       widget->primary_color[0],
    "green",     widget->primary_color[1],
    "blue",      widget->primary_color[2],
    "alpha",     65536,
    NULL);
  gegl_node_connect (rect, "input", prev, "output");
  prev = rect;

  rect = gegl_graph_create_node (gegl,
    "operation", "OpFillRect",
    "in-place",  TRUE,
    "x",         (x-(mini_box_size-2 )/2),
    "y",         (y+(bar_height/2)+1),
    "width",     (mini_box_size-2),
    "height",    (mini_box_size-2),
    "red",       widget->secondary_color[0],
    "green",     widget->secondary_color[1],
    "blue",      widget->secondary_color[2],
    "alpha",     65536,
    NULL);
  gegl_node_connect (rect, "input", prev, "output");

  gegl_node_apply (rect, "output");
  
  g_object_unref (gegl);
}

static gboolean
event (View *view,
       gint x,
       gint y,
       gint pressure,
       gulong time,
       gpointer data)
{
  Slider *slider = (Slider*) view;
  Box *box = (Box*) view;

  gint old_value = slider->value;
  gint length = box_get_width (box);

  gint fraction = x;
  slider->value = MAX (0, MIN ((fraction / length), SLIDER_MAX));
  
  /* value can change in below func. */
  if (slider->value != old_value)
    {
      if (slider->value_changed_function != NULL)
        slider->value_changed_function (slider, slider->data);
    }
  
   
  return TRUE;
}


void
slider_set_value_changed_function (Slider *slider,
                                   void  (*value_changed_function) (Slider    *slider,
	                                                            gpointer  data),
	                           gpointer  data)
{
  slider->value_changed_function = value_changed_function;
  slider->data = data;
}

Slider *
slider_new (Canvas   *canvas)
{
  Slider *slider = g_malloc0 (sizeof (Slider));
  Widget *widget = (Widget*) slider;
  View   *view   = (View*) widget;

  widget_init (widget, canvas);
  
  view_set_event (view, event, slider);
  view_set_render (view, render, NULL);
 
  slider->old_value = -1; 
  
  slider->allocation = canvas_alloc (canvas, 250, 250);
  
  view_lookat (view, view->canvas,
     CANVAS_RECT (slider->allocation)->x, CANVAS_RECT (slider->allocation)->y, 65536);
  return slider;
}

void 
slider_set_value (Slider *slider,
		  guint   value)
{
  slider->value = value;
}

guint
slider_get_value (Slider *slider)
{
  return slider->value; 
}
